@@ -62,24 +62,26 @@ describe("LocalWorkspace", () => {
     }));
 
     it(`create/select/remove LocalWorkspace stack`, asyncTest(async () => {
+        const projectName = "node_test";
         const projectSettings: ProjectSettings = {
-            name: "node_test",
+            name: projectName,
             runtime: "nodejs",
         };
         const ws = await LocalWorkspace.create({ projectSettings });
-        const stackName = `int_test${getTestSuffix()}`;
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         await ws.createStack(stackName);
         await ws.selectStack(stackName);
         await ws.removeStack(stackName);
     }));
 
     it(`create/select/createOrSelect Stack`, asyncTest(async () => {
+        const projectName = "node_test";
         const projectSettings: ProjectSettings = {
-            name: "node_test",
+            name: projectName,
             runtime: "nodejs",
         };
         const ws = await LocalWorkspace.create({ projectSettings });
-        const stackName = `int_test${getTestSuffix()}`;
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         await Stack.create(stackName, ws);
         await Stack.select(stackName, ws);
         await Stack.createOrSelect(stackName, ws);
@@ -92,7 +94,7 @@ describe("LocalWorkspace", () => {
             runtime: "nodejs",
         };
         const ws = await LocalWorkspace.create({ projectSettings });
-        const stackName = `int_test${getTestSuffix()}`;
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await Stack.create(stackName, ws);
 
         const config = {
@@ -130,7 +132,10 @@ describe("LocalWorkspace", () => {
         await ws.removeStack(stackName);
     }));
     it(`nested_config`, asyncTest(async () => {
-        const stackName = fullyQualifiedStackName("pulumi-test", "nested_config", "dev");
+        if (getTestOrg() !== "pulumi-test") {
+            return;
+        }
+        const stackName = fullyQualifiedStackName(getTestOrg(), "nested_config", "dev");
         const workDir = upath.joinSafe(__dirname, "data", "nested_config");
         const stack = await LocalWorkspace.createOrSelectStack({ stackName, workDir });
 
@@ -152,15 +157,16 @@ describe("LocalWorkspace", () => {
         assert.strictEqual(list.value, "[\"one\",\"two\",\"three\"]");
     }));
     it(`can list stacks and currently selected stack`, asyncTest(async () => {
+        const projectName = `node_list_test${getTestSuffix()}`;
         const projectSettings: ProjectSettings = {
-            name: `node_list_test${getTestSuffix()}`,
+            name: projectName,
             runtime: "nodejs",
         };
         const ws = await LocalWorkspace.create({ projectSettings });
         const stackNamer = () => `int_test${getTestSuffix()}`;
         const stackNames: string[] = [];
         for (let i = 0; i < 2; i++) {
-            const stackName = stackNamer();
+            const stackName = fullyQualifiedStackName(getTestOrg(), projectName, stackNamer());
             stackNames[i] = stackName;
             await Stack.create(stackName, ws);
             const stackSummary = await ws.stack();
@@ -174,12 +180,13 @@ describe("LocalWorkspace", () => {
         }
     }));
     it(`stack status methods`, asyncTest(async () => {
+        const projectName = "node_test";
         const projectSettings: ProjectSettings = {
-            name: "node_test",
+            name: projectName,
             runtime: "nodejs",
         };
         const ws = await LocalWorkspace.create({ projectSettings });
-        const stackName = `int_test${getTestSuffix()}`;
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await Stack.create(stackName, ws);
         const history = await stack.history();
         assert.strictEqual(history.length, 0);
@@ -188,7 +195,7 @@ describe("LocalWorkspace", () => {
         await ws.removeStack(stackName);
     }));
     it(`runs through the stack lifecycle with a local program`, asyncTest(async () => {
-        const stackName = `int_test${getTestSuffix()}`;
+        const stackName = fullyQualifiedStackName(getTestOrg(), "testproj", `int_test${getTestSuffix()}`);
         const workDir = upath.joinSafe(__dirname, "data", "testproj");
         const stack = await LocalWorkspace.createStack({ stackName, workDir });
 
@@ -235,8 +242,8 @@ describe("LocalWorkspace", () => {
                 exp_secret: config.getSecret("buzz"),
             };
         };
-        const stackName = `int_test${getTestSuffix()}`;
         const projectName = "inline_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
 
         const stackConfig: ConfigMap = {
@@ -282,8 +289,8 @@ describe("LocalWorkspace", () => {
                 exp_secret: config.getSecret("buzz"),
             };
         };
-        const stackName = `int_test${getTestSuffix()}`;
         const projectName = "inline_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
 
         const stackConfig: ConfigMap = {
@@ -333,6 +340,166 @@ describe("LocalWorkspace", () => {
 
         await stack.workspace.removeStack(stackName);
     }));
+    it(`has secret config warnings`, asyncTest(async () => {
+        const program = async () => {
+            const config = new Config();
+
+            config.get("plainstr1");
+            config.require("plainstr2");
+            config.getSecret("plainstr3");
+            config.requireSecret("plainstr4");
+
+            config.getBoolean("plainbool1");
+            config.requireBoolean("plainbool2");
+            config.getSecretBoolean("plainbool3");
+            config.requireSecretBoolean("plainbool4");
+
+            config.getNumber("plainnum1");
+            config.requireNumber("plainnum2");
+            config.getSecretNumber("plainnum3");
+            config.requireSecretNumber("plainnum4");
+
+            config.getObject("plainobj1");
+            config.requireObject("plainobj2");
+            config.getSecretObject("plainobj3");
+            config.requireSecretObject("plainobj4");
+
+            config.get("str1");
+            config.require("str2");
+            config.getSecret("str3");
+            config.requireSecret("str4");
+
+            config.getBoolean("bool1");
+            config.requireBoolean("bool2");
+            config.getSecretBoolean("bool3");
+            config.requireSecretBoolean("bool4");
+
+            config.getNumber("num1");
+            config.requireNumber("num2");
+            config.getSecretNumber("num3");
+            config.requireSecretNumber("num4");
+
+            config.getObject("obj1");
+            config.requireObject("obj2");
+            config.getSecretObject("obj3");
+            config.requireSecretObject("obj4");
+        };
+        const projectName = "inline_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
+        const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
+
+        const stackConfig: ConfigMap = {
+            "plainstr1": { value: "1" },
+            "plainstr2": { value: "2" },
+            "plainstr3": { value: "3" },
+            "plainstr4": { value: "4" },
+            "plainbool1": { value: "true" },
+            "plainbool2": { value: "true" },
+            "plainbool3": { value: "true" },
+            "plainbool4": { value: "true" },
+            "plainnum1": { value: "1" },
+            "plainnum2": { value: "2" },
+            "plainnum3": { value: "3" },
+            "plainnum4": { value: "4" },
+            "plainobj1": { value: "{}" },
+            "plainobj2": { value: "{}" },
+            "plainobj3": { value: "{}" },
+            "plainobj4": { value: "{}" },
+            "str1": { value: "1", secret: true },
+            "str2": { value: "2", secret: true },
+            "str3": { value: "3", secret: true },
+            "str4": { value: "4", secret: true },
+            "bool1": { value: "true", secret: true },
+            "bool2": { value: "true", secret: true },
+            "bool3": { value: "true", secret: true },
+            "bool4": { value: "true", secret: true },
+            "num1": { value: "1", secret: true },
+            "num2": { value: "2", secret: true },
+            "num3": { value: "3", secret: true },
+            "num4": { value: "4", secret: true },
+            "obj1": { value: "{}", secret: true },
+            "obj2": { value: "{}", secret: true },
+            "obj3": { value: "{}", secret: true },
+            "obj4": { value: "{}", secret: true },
+        };
+        await stack.setAllConfig(stackConfig);
+
+        let events: string[] = [];
+        const findDiagnosticEvents = (event: EngineEvent) => {
+            if (event.diagnosticEvent?.severity === "warning") {
+                events.push(event.diagnosticEvent.message);
+            }
+        };
+
+        const expectedWarnings = [
+            "Configuration 'inline_node:str1' value is a secret; use `getSecret` instead of `get`",
+            "Configuration 'inline_node:str2' value is a secret; use `requireSecret` instead of `require`",
+            "Configuration 'inline_node:bool1' value is a secret; use `getSecretBoolean` instead of `getBoolean`",
+            "Configuration 'inline_node:bool2' value is a secret; use `requireSecretBoolean` instead of `requireBoolean`",
+            "Configuration 'inline_node:num1' value is a secret; use `getSecretNumber` instead of `getNumber`",
+            "Configuration 'inline_node:num2' value is a secret; use `requireSecretNumber` instead of `requireNumber`",
+            "Configuration 'inline_node:obj1' value is a secret; use `getSecretObject` instead of `getObject`",
+            "Configuration 'inline_node:obj2' value is a secret; use `requireSecretObject` instead of `requireObject`",
+        ];
+
+        // These keys should not be in any warning messages.
+        const unexpectedWarnings = [
+            "plainstr1",
+            "plainstr2",
+            "plainstr3",
+            "plainstr4",
+            "plainbool1",
+            "plainbool2",
+            "plainbool3",
+            "plainbool4",
+            "plainnum1",
+            "plainnum2",
+            "plainnum3",
+            "plainnum4",
+            "plainobj1",
+            "plainobj2",
+            "plainobj3",
+            "plainobj4",
+            "str3",
+            "str4",
+            "bool3",
+            "bool4",
+            "num3",
+            "num4",
+            "obj3",
+            "obj4",
+        ];
+
+        const validate = (warnings: string[]) => {
+            for (const expected of expectedWarnings) {
+                let found = false;
+                for (const warning of warnings) {
+                    if (warning.includes(expected)) {
+                        found = true;
+                        break;
+                    }
+                }
+                assert.strictEqual(found, true, `expected warning not found`);
+            }
+            for (const unexpected of unexpectedWarnings) {
+                for (const warning of warnings) {
+                    assert.strictEqual(warning.includes(unexpected), false,
+                        `Unexpected '${unexpected}' found in warning`);
+                }
+            }
+        };
+
+        // pulumi preview
+        await stack.preview({ onEvent: findDiagnosticEvents });
+        validate(events);
+
+        // pulumi up
+        events = [];
+        await stack.up({ onEvent: findDiagnosticEvents });
+        validate(events);
+
+        await stack.workspace.removeStack(stackName);
+    }));
     it(`imports and exports stacks`, asyncTest(async() => {
         const program = async () => {
             const config = new Config();
@@ -342,8 +509,8 @@ describe("LocalWorkspace", () => {
                 exp_secret: config.getSecret("buzz"),
             };
         };
-        const stackName = `int_test${getTestSuffix()}`;
         const projectName = "import_export_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
 
         try {
@@ -376,8 +543,8 @@ describe("LocalWorkspace", () => {
                 exp_secret: config.getSecret("buzz"),
             };
         };
-        const stackName = `int_test${getTestSuffix()}`;
         const projectName = "import_export_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
 
         const assertOutputs = (outputs: OutputMap) => {
@@ -423,8 +590,8 @@ describe("LocalWorkspace", () => {
             Promise.reject(new Error());
             return {};
         };
-        const stackName = `int_test${getTestSuffix()}`;
         const projectName = "inline_node";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack({ stackName, projectName, program });
 
         // pulumi up
@@ -443,14 +610,15 @@ describe("LocalWorkspace", () => {
         assert.strictEqual(versionRegex.test(ws.pulumiVersion), true);
     }));
     it(`respects existing project settings`, asyncTest(async () => {
-        const stackName = `int_test${getTestSuffix()}`;
-        const projectName = "project_was_overwritten";
+        const projectName = "correct_project";
+        const stackName = fullyQualifiedStackName(getTestOrg(), projectName, `int_test${getTestSuffix()}`);
         const stack = await LocalWorkspace.createStack(
             {stackName, projectName, program: async() => { return; }},
             {workDir: upath.joinSafe(__dirname, "data", "correct_project")},
         );
         const projectSettings = await stack.workspace.projectSettings();
         assert.strictEqual(projectSettings.name, "correct_project");
+        // the description check is enough to verify that the stack wasn't overwritten
         assert.strictEqual(projectSettings.description, "This is a description");
         await stack.workspace.removeStack(stackName);
     }));
@@ -469,13 +637,15 @@ describe("LocalWorkspace", () => {
                 "PULUMI_CONFIG_PASSPHRASE": "test",
             },
         });
+        for (let i = 0; i < stacks.length; i++) {
+            await Stack.create(stacks[i], ws);
+        }
         for (let i = 0; i < stacks.length; i++) {
             const x = i;
             const s = stacks[i];
             dones.push((async () => {
-                const stack = await Stack.createOrSelect(s, ws);
                 for (let j = 0; j < 20; j++) {
-                    await stack.setConfig("var-" + j, { value: ((x*20)+j).toString() });
+                    await ws.setConfig(s, "var-" + j, { value: ((x*20)+j).toString()});
                 }
             })());
         }
@@ -588,3 +758,11 @@ const normalizeConfigKey = (key: string, projectName: string) => {
     }
     return "";
 };
+
+const getTestOrg = () => {
+    let testOrg = "pulumi-test";
+    if (process.env.PULUMI_TEST_ORG) {
+        testOrg = process.env.PULUMI_TEST_ORG;
+    }
+    return testOrg;
+};
